from typing import Any, Callable


# noinspection PyPep8Naming
def DataspreeInferenceType(name: str, super_type: type, constraint: Callable[[Any], bool]):
    def dataspree_inference_type(x: Any) -> bool:
        return constraint(x)

    setattr(dataspree_inference_type, '__name__', name)
    setattr(dataspree_inference_type, '__supertype__', super_type)
    return dataspree_inference_type


from functools import partial


class FilterReference:
    def __init__(self, app: 'Application', filter_name: str):
        self.__callee__: Callable[[], 'LinkFilter'] = partial(app.filters.get, filter_name)

    def __call__(self) -> 'LinkFilter':
        return self.__callee__()


class NodeReference:
    def __init__(self, app: 'Application', node_name: str):
        self.__callee__: Callable[[], 'Node'] = partial(app.nodes.get, node_name)

    def __call__(self) -> 'Node':
        return self.__callee__()


class VariableReference:
    def __init__(self, app: 'Application', var_name: str):
        self.__callee__: Callable[[], Any] = partial(app.getattr, var_name)

    def __call__(self) -> 'Any':
        return self.__callee__()
